import {confirm, EventData, NavigatedData, Page} from "@nativescript/core";
import {firebase} from '@nativescript/firebase-core';
import {ToastDuration, ToastPosition, Toasty} from "@triniwiz/nativescript-toasty";
import {vm} from "./main-view-model";
import {GoogleAuthProvider, PhoneAuthProvider, UserCredential} from "@nativescript/firebase-auth";
import {GoogleSignin} from "@nativescript/google-signin";

let page: Page;


export function onPageNavigatingTo(args: NavigatedData) {
  page = args.object as Page;
  page.bindingContext = vm;
}


export function submitRegisterEmail(args: EventData) {
  firebase().auth().createUserWithEmailAndPassword(vm.get("emailSignUp"), vm.get("passwordEmailSignUp")).then((result: UserCredential) => {
    console.log("User created: " + JSON.stringify(result.user));

    result.user.sendEmailVerification().then(() => {
      alert({
        title: null,
        message: "Verification email sent!",
        okButtonText: "Ok"
      });
    }, (e) => {
      alert({
        title: "Error sending email verification",
        message: e ? e.toString() : "",
        okButtonText: "Ok"
      });
    });
  }, (e) => {
    console.log("Error creating user: " + e);

    alert({
      title: "User not registered :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });
}


export function submitLoginEmailPassword(args: EventData) {
  firebase().auth().signInWithEmailAndPassword(vm.get("emailLogin"), vm.get("passwordEmailLogin")).then((result: UserCredential) => {
    console.log("Password login OK: " + JSON.stringify(result));

    alert({
      title: "Login succeeded",
      message: JSON.stringify(result),
      okButtonText: "Ok"
    });
  }, (e) => {
    console.log("Login error: " + e);

    alert({
      title: "Login error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });
}


export function doSendPasswordResetEmail() {
  firebase().auth().sendPasswordResetEmail(vm.get("emailLogin")).then(() => {
    showToast("Password reset email has been sent.", true);
  }, (e) => {
    console.log("Reset password error: " + e);

    alert({
      title: "Reset password error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });
}


export function submitLoginPhone() {
  PhoneAuthProvider.provider().verifyPhoneNumber(vm.get("phoneLogin")).then((verificationId: string) => {
    // present ui to allow user to enter verificationCode
    // use the verificationCode entered by the user to create PhoneAuthCredentials
    /*const credential = PhoneAuthProvider.provider().credential(verificationId, verificationCode);
    firebase().auth().signInWithCredential(credential);*/

    console.log("verifyPhoneNumber received verification id:");
    console.log(verificationId);
  }, (e) => {
    console.log("Login error: " + e);

    alert({
      title: "Login error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });
}


export function doGoogleSignIn() {
  GoogleSignin.configure({
    scopes: [
      "https://www.googleapis.com/auth/contacts.readonly",
      "https://www.googleapis.com/auth/user.birthday.read"
    ]
  }).then(() => {
    console.log("############### GoogleSignIn configured :)");

    GoogleSignin.signIn().then((user) => {
      const credential = GoogleAuthProvider.credential(user.accessToken, user.idToken);
      firebase().auth().signInWithCredential(credential).then((result: UserCredential) => {
        console.log("Google login OK: " + JSON.stringify(result));

        alert({
          title: "Google login succeeded",
          message: JSON.stringify(result),
          okButtonText: "Ok"
        });
      }, (e) => {
        console.log(":(");
      });
    }, (e) => {
      alert({
        title: "Google login error",
        message: e ? e.toString() : "",
        okButtonText: "Ok"
      });
    });
  }, (e) => {
    console.log("############### GoogleSignIn configure error:");
    console.log(e);
  });

  /*firebase.login({
    type: firebase.LoginType.GOOGLE,
    googleOptions: {
      scopes: [
        "https://www.googleapis.com/auth/contacts.readonly",
        "https://www.googleapis.com/auth/user.birthday.read"
      ]
    }
  }).then((result) => {
      console.log("Google login OK: " + JSON.stringify(result));

      alert({
        title: "Google login succeeded",
        message: JSON.stringify(result),
        okButtonText: "Ok"
      });
    }, (e) => {
      alert({
        title: "Google login error",
        message: e ? e.toString() : "",
        okButtonText: "Ok"
      });
    }
  );*/
}


export function doAppleSignIn() {
    /*firebase.login({
        type: firebase.LoginType.APPLE,
        appleOptions: {
            locale: "de",
            scopes: ["email"]
        }
    }).then((result) => {
        console.log("Apple login OK: " + JSON.stringify(result))
    }, (e) => {
        console.log(e);
        alert({
            title: "Login error :(",
            message: e ? e.toString() : "",
            okButtonText: "Ok"
        });
    });*/
}


export function doDeleteUser() {
  confirm("Do you really want to delete the current user from firebase?").then((confirmed: boolean) => {
    if(confirmed === true) {
      /*firebase.deleteUser().then(() => {
        showToast("User successfully deleted");
      }, (e) => {
        alert({
          title: "Delete user error :(",
          message: e ? e.toString() : "",
          okButtonText: "Ok"
        });
      });*/
    }
  });
}


export function doLogout() {
  firebase().auth().signOut();
}


export function doUpdateEmail() {
  /*firebase.updateEmail(vm.get("emailUpdate")).then(() => {
    showToast("Email updated", true);
  }, (e) => {
    console.log(e);

    alert({
      title: "Update email error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });*/
}


export function doUpdatePassword() {
  /*firebase.updatePassword(vm.get("passwordUpdate")).then(() => {
    showToast("Password updated", true);
  }, (e) => {
    console.log(e);

    alert({
      title: "Update password error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });*/
}


export function getCurrentUser(args: EventData) {
  /*firebase.getCurrentUser().then((user: firebase.User) => {
    console.log("Current user: " + JSON.stringify(user));

    alert({
      title: "Current user",
      message: JSON.stringify(user),
      okButtonText: "Ok"
    });
  }, (e) => {
    alert({
      title: "Current user error :(",
      message: e ? e.toString() : "",
      okButtonText: "Ok"
    });
  });*/
}


function showToast(message: string, showLong?: boolean) {
  try {
    const toast = new Toasty({ text : message })
      .setToastDuration(showLong === true ? ToastDuration.LONG : ToastDuration.SHORT)
      .setToastPosition(ToastPosition.BOTTOM)
      .setTextColor("#FFFFFF")
      .setBackgroundColor("#656565");

    toast.yAxisOffset = 10;

    toast.show();
  } catch (e) {
    console.log("Error showToast:");
    console.log(e);

    alert({
      title: null,
      message: message,
      okButtonText: "Ok"
    });
  }
}
